-- AngelPlayer: Battlemod support
-- Enables a Chaos Energy powered 'Howling' to better fight against enemies with more elaborate behaviours.

local battoru_set
local HOWLING_LIMIT = 4*TICRATE		-- Higher values may cause strain on the user.

rawset(_G, "AP_BattleMod", function()

	local b = CBW_Battle
	if not b then return end		-- Mod not loaded
	if battoru_set then return end	-- Mod already loaded in

	b.SkinVars["kanade"] = {

		flags = SKINVARS_GUARD|SKINVARS_NOSPINSHIELD,
		weight = 100,
		shields = 1,	-- Offence axed.

		special =	function(mo, act)

						local p = mo.player

						p.actiontext = "Guard Skill: Howling"
						p.actionrings = 10

						-- (unique guard frame in AP_Extras)
						-- Hand Sonic priority:
						if mo.ap_handstate == HS_DASH
							CBW_Battle.SetPriority(p, 2, 1, "knuckles_glide", 2, 1, "Hand Sonic")
							-- Use Knuckles glide check / with different priorities.
						end
						
						local function stop_howling(p)
							b.ApplyCooldown(p, TICRATE*20)
							p.actiontime = 0
							p.actionstate = 0
							if not P_PlayerInPain(p)
								p.mo.state = S_PLAY_STND
							end	
							--p.powers[pw_nocontrol] = 0
							AP_hresetStats(p)
						end
						

						-- button press garbage
						if not p.actionstate and act == 1
						and P_IsObjectOnGround(mo)
							p.actionstate = 1
							p.actiontime = 1
							b.PayRings(p)

							p.ap_hsavestats = {p.normalspeed, p.thrustfactor, p.jumpfactor}
							p.normalspeed, p.thrustfactor, p.jumpfactor = 0,0,0
							
						end
						if p.actionstate == 1		-- Howling init
							-- readying attack:
							mo.state = S_PLAY_HOWLING_PREP1
							--p.powers[pw_nocontrol] = 2
							p.actionstate = 2
							p.actiontext = "Charging..."
							
							if P_PlayerInPain(p)
							or not P_IsObjectOnGround(mo)
								stop_howling(p)
							end	

						elseif p.actionstate == 2	-- Howling prep anim
							--p.powers[pw_nocontrol] = 2
							p.actiontext = "Charging..."

							p.actiontime = $+1
							if mo.state != S_PLAY_HOWLING_PREP1
							and mo.state != S_PLAY_HOWLING_PREP2	-- bruh
							and mo.state != S_PLAY_HOWLING
								mo.state = S_PLAY_HOWLING_PREP2
								mo.tics = max(1, $-p.actiontime)	-- mercy mechanic on this
							end


							if mo.state == S_PLAY_HOWLING	-- prep finished
								p.actionstate = 3
								p.actiontime = 1	-- reset actiontime.
								P_FlashPal(p, 1, 1)
								if p == consoleplayer
									P_StartQuake(FRACUNIT*64, 6)
								end
								S_StartSound(mo, sfx_howl1)
							end

							if P_PlayerInPain(p)
							or not P_IsObjectOnGround(mo)
								stop_howling(p)
							end	

						elseif p.actionstate == 3	-- Howling proper!
							local duration = HOWLING_LIMIT - p.actiontime
							local duration_1 = tostring(G_TicsToSeconds(duration))
							local duration_2 = tostring(G_TicsToCentiseconds(duration))
							
							p.actiontext = "Hold to keep firing ("..duration_1.."."..duration_2..")"
							p.actiontime = $+1
							--p.powers[pw_nocontrol] = 3

							p.drawangle = mo.angle	-- keep updating this.

							if p.actiontime%10 == 0
							and p.actiontime > 5
								S_StartSound(mo, sfx_howl2)
							end

							if p == consoleplayer
							and p.actiontime > 5
								P_StartQuake(FRACUNIT*20, 6)
							end

							if p.actiontime%10 == 0
								mo.height = $*3	-- help missile spawning.
								local m = P_SPMAngle(mo, MT_HOWLING, mo.angle, 0)
								mo.height = $/3
								if m and m.valid
									m.color = mo.color
									m.angle = $ + ANG1*90
									m.scale = $*2
									m.destscale = mo.scale*8
									m.scalespeed = mo.scale/12
								end
							end

							local actp = p.cmd.buttons & BT_ATTACK

							-- give 0.5 sec of margin to use the ability with only 20 rings.
							if not actp and p.actiontime > TICRATE*3/2
							or p.actiontime > HOWLING_LIMIT
							or P_PlayerInPain(p)
							or not P_IsObjectOnGround(mo)
								
								S_StopSound(mo)
								
								if not P_PlayerInPain(p)
								and P_IsObjectOnGround(mo)
									S_StartSound(mo, sfx_bkpoof)
									mo.height = $*3	-- help missile spawning.
									local m = P_SPMAngle(mo, MT_HOWLING, mo.angle, 0)
									mo.height = $/3
									if m and m.valid
										m.momx = $*3/2
										m.momy = $*3/2
										m.momz = $*3/2
										m.color = mo.color
										m.angle = $ + ANG1*90
										m.scale = $*2
										m.destscale = mo.scale*8
										m.scalespeed = mo.scale/12
										m.tumbles = true
									end
								end

								stop_howling(p)
							end
						end
					end,

	}

	battoru_set = true
	print("Angel_Player: 'BattleMod' library loaded succesfully. 'Howling' is now available in Battle modes.")
end)